@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -92,6 +92,14 @@ int Basic_buffer::characters( const int line ) const throw()
   }
 
 
+bool Basic_buffer::compare_lines( const int line1, const int line2 ) const throw()
+  {
+  if( line1 < 0 || line1 >= lines() || line2 < 0 || line2 >= lines() )
+    return false;
+  return ( data[line1] == data[line2] );
+  }
+
+
 bool Basic_buffer::one_character() const throw()
   {
   return ( data[0].size() == 1 &&
@@ -265,6 +273,34 @@ bool Basic_buffer::pdelb( const Point & p1, const Point & p2 ) throw()
   }
 
 
+bool Basic_buffer::to_lowercase( const Point & p1, const Point & p2 ) throw()
+  {
+  bool change = false;
+  if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
+    for( Point p = p1; p < p2; pnext( p ) )
+      {
+      const unsigned char ch = data[p.line][p.col];
+      if( ISO_8859::isupper( ch ) )
+        { data[p.line][p.col] = ISO_8859::tolower( ch ); change = true; }
+      }
+  return change;
+  }
+
+
+bool Basic_buffer::to_uppercase( const Point & p1, const Point & p2 ) throw()
+  {
+  bool change = false;
+  if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
+    for( Point p = p1; p < p2; pnext( p ) )
+      {
+      const unsigned char ch = data[p.line][p.col];
+      if( ISO_8859::islower( ch ) )
+        { data[p.line][p.col] = ISO_8859::toupper( ch ); change = true; }
+      }
+  return change;
+  }
+
+
 // Is p at beginning of word?
 bool Basic_buffer::pisbow( const Point & p ) const throw()
   {
@@ -389,9 +425,12 @@ int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
     return 0;
     }
 
-  { Point pt = p;		// no match if delimiter is escaped
-    if( ppgetc( pt ) == '\\' )
-      { Point pt2 = pt; if( ppgetc( pt2 ) != '\\' ) return -1; } }
+  {
+  Point pt = p;		// no match if delimiter is escaped
+  if( ppgetc( pt ) == '\\' )
+    { Point pt2 = pt; if( ppgetc( pt2 ) != '\\' ) return -1; }
+  }
+
   Point p1 = p;
   int level = 0;	// nesting level of delimiters
   bool quote = false;	// ignore double-quoted text
