@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,13 +19,13 @@
 #include <cerrno>
 #include <cstdio>
 #include <cstdlib>
-#include <ctime>
 #include <string>
 #include <vector>
 #include <unistd.h>
 #include <sys/stat.h>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "block.h"
 #include "iso_8859.h"
 #include "menu.h"
@@ -33,12 +33,6 @@
 #include "screen.h"
 
 
-// Main data structure of the editor.
-// Contains the handles to all the buffers being edited.
-//
-Bufhandle_vector bufhandle_vector;
-
-
 namespace {
 
 const std::string unnamed( "(Unnamed)" );
@@ -49,9 +43,9 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b ) throw()
   static std::vector< Point * > v;
   v.clear();
   for( int i = 0; i < 10; ++i ) v.push_back( &b.mark[i] );
-  for( int i = 0; i < bufhandle_vector.handles(); ++i )
+  for( int i = 0; i < Bufhandle_vector::handles(); ++i )
     {
-    Buffer_handle & bh = bufhandle_vector[i];
+    Buffer_handle & bh = Bufhandle_vector::handle( i );
     if( &b == &bh.buffer() ) v.push_back( &bh.pointer );
     }
   if( Block::bufferp() == &b )
@@ -158,7 +152,7 @@ bool Buffer::Options::set_rmargin( const int r ) throw()
 
 
 Buffer::Buffer( const Options & opts, const std::string * namep ) throw( Buffer::Error )
-  : crlf( false ), _modified( false ), saved( false ), options( opts )
+  : _modified( false ), saved( false ), options( opts ), crlf( false )
   {
   FILE *f = 0;
   errno = 0;
@@ -234,7 +228,7 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
     if( options.read_only ) return false;
     if( !saved && RC::editor_options().backup )
       {
-      std::string backname = *namep; backname += '~';
+      std::string backname( *namep ); backname += '~';
       std::remove( backname.c_str() );
       std::rename( namep->c_str(), backname.c_str() );
       }
@@ -277,19 +271,24 @@ const std::string & Buffer::character_info( const Point & p ) const throw()
   int ch = (*this)[p];
 
   if( ch >= 0 )
-    snprintf( tmp2, sizeof( tmp2 ), "Code %d(0x%X/0%o) **", ch, ch ,ch );
+    snprintf( tmp2, sizeof( tmp2 ), "Code %d(0x%02X/0%o) **", ch, ch ,ch );
   else if( p == eof() )
     snprintf( tmp2, sizeof( tmp2 ), "(End Of File) **" );
   else
     { snprintf( tmp2, sizeof( tmp2 ), "*** Error: Bad Pointer: Line %d  Col %d",
                 p.line + 1, p.col + 1 ); s = tmp2; return s; }
 
-  int offset = p.col;
-  for( int i = 0; i < p.line; ++i ) offset += characters( i );
+  int percent = 0;
+  for( int i = 0; i < p.line; ++i ) percent += characters( i );
+  int offset = percent + p.col;
   if( crlf ) { offset += p.line; if( ch == '\n' ) ++offset; }
+  for( int i = p.line; i < eof().line; ++i ) percent += characters( i );
+  if( crlf ) percent += eof().line;
+  percent += eof().col;
+  if( percent > 0 ) percent = ( 100LL * offset ) / percent;
 
-  snprintf( tmp1, sizeof( tmp1 ), "** Line %d  Col %d  Offset %d(0x%X)  ",
-            p.line + 1, p.col + 1, offset, offset );
+  snprintf( tmp1, sizeof( tmp1 ), "** Line %d  Col %d  Offset %d(0x%X)  %d%%  ",
+            p.line + 1, p.col + 1, offset, offset, percent );
   s = tmp1; s += tmp2;
   return s;
   }
@@ -385,9 +384,9 @@ bool Buffer::pputc( Point & p, const unsigned char ch ) throw()
         {
         const Point pt = bot( p0 );
         Point p2 = p0;
-        while( p2 > pt && pprev( p2 ) && !ISO_8859::isblank( (*this)[p2] ) );
+        while( p2 > pt && pprev( p2 ) && !ISO_8859::isblank( (*this)[p2] ) ) ;
         Point p1 = p2;
-        while( p1 > pt && pprev( p1 ) && ISO_8859::isblank( (*this)[p1] ) );
+        while( p1 > pt && pprev( p1 ) && ISO_8859::isblank( (*this)[p1] ) ) ;
         if( p1 >= pt && p1 < p2 && pnext( p1 ) && pnext( p2 ) )
           {
           reset_appendable(); pdelb( p1, p2 ); p2 = p1;
@@ -530,137 +529,3 @@ bool Buffer::redo( Point & p ) throw()
   std::swap( _modified, rec.modified );
   return true;
   }
-
-
-// Saves modified buffers if editor dies
-//
-Bufhandle_vector::~Bufhandle_vector() throw()
-  {
-  if( handles() == 0 ) return;
-  int i;
-  for( i = 0; i < handles(); ++i )
-    if( data[i].buffer().modified() ) break;
-  if( i >= handles() ) return;
-
-  const std::string dead_name = "DEADMOE";
-  RC::editor_options().backup = false;
-  FILE *f = std::fopen( dead_name.c_str(), "a" );
-  time_t dummy = time( 0 );
-  std::fprintf( f, "\n*** Modified files in Moe when it aborted on %s", ctime( &dummy ) );
-  std::fflush( f );
-  for( i = 0; i < handles(); ++i )
-    if( data[i].buffer().modified() )
-      {
-      std::fprintf( f, "\n*** File '%s'\n", data[i].buffer().uname().c_str() );
-      std::fflush( f );
-      data[i].buffer().save( &dead_name, false, true );
-      }
-  std::fclose( f );
-  }
-
-
-void Bufhandle_vector::save_all_named() const throw()
-  {
-  int count = 0;
-  bool done = true;
-
-  for( int i = 0; i < handles(); ++i )
-    {
-    const Buffer & buffer = data[i].buffer();
-    if( buffer.modified() && buffer.name().size() )
-      {
-      try { if( buffer.save() ) ++count; else done = false; }
-      catch( Buffer::Error ) { done = false; }
-      }
-    }
-  const char * s = ( done ? "" : "Only " );
-  char buf[80];
-  snprintf( buf, sizeof( buf ), "%s%d named buffer(s) saved", s, count );
-  Screen::show_status_lines();
-  Screen::show_message( buf );
-  }
-
-
-void Bufhandle_vector::show_status() const throw()
-  {
-  int mcount = 0, ucount = 0;
-
-  for( int i = 0; i < handles(); ++i )
-    {
-    if( data[i].buffer().modified() ) ++mcount;
-    if( !data[i].buffer().name().size() ) ++ucount;
-    }
-
-  char buf[80];
-  snprintf( buf, sizeof( buf ),
-            "**  %d modified and  %d unnamed out of  %d total buffer(s) **",
-            mcount, ucount, handles() );
-  Screen::show_status_lines();
-  Screen::show_message( buf );
-  }
-
-
-int Bufhandle_vector::handles( const Buffer & buffer ) const throw()
-  {
-  int c = 0;
-  for( int i = 0; i < handles(); ++i ) if( &buffer == &data[i].buffer() ) ++c;
-  return c;
-  }
-
-
-int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
-                                          const std::string * namep,
-                                          int i, const int line ) throw( Buffer::Error )
-  {
-  if( namep && namep->size() )
-    for( int j = 0; j < handles(); ++j )
-      if( *namep == data[j].buffer().name() )
-        return circular_index( j );
-
-  Buffer *bufp = new Buffer( opts, namep );
-  if( namep && namep->size() ) RC::apply_regex_options( *bufp );
-
-  if( i < 0 ) i = handles();
-  else
-    {
-    i = absolute_index( i );
-    if( i == 0 ) i = handles(); else if( i <= _first ) ++_first;
-    }
-  data.insert( data.begin() + i, Buffer_handle( *bufp, line ) );
-  return circular_index( i );
-  }
-
-
-     // Add empty buffer if no files given or stdin is not a tty
-void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line ) throw( Buffer::Error )
-  {
-  if( !handles() || !isatty( fileno( stdin ) ) )
-    find_or_add_handle( RC::default_buffer_options(), 0, -1, line );
-  }
-
-
-void Bufhandle_vector::duplicate_handle( int i ) throw()
-  {
-  i = absolute_index( i );
-  if( i < _first ) ++_first;
-  data.insert( data.begin() + i + 1, data[i] );
-  }
-
-
-bool Bufhandle_vector::delete_handle( int i ) throw()
-  {
-  if( i < 0 || i >= handles() ) return false;
-  else i = absolute_index( i );
-  Buffer & buffer = data[i].buffer();
-  int j;
-  for( j = 0; j < handles(); ++j )
-    if( j != i && &buffer == &data[j].buffer() ) break;
-  if( j >= handles() )			// last handle for this buffer
-    {
-    if( Block::bufferp() == &buffer ) Block::reset();
-    delete &buffer;
-    }
-  data.erase( data.begin() + i );
-  if( i <= _first && handles() ) prev();
-  return true;
-  }
