@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,17 +21,17 @@
     (eg, bug) which caused moe to panic.
 */
 
-//#include <clocale>
 #include <cctype>
 #include <csignal>
 #include <cstdio>
 #include <cstdlib>
-#include <ctime>
+#include <cstring>
 #include <string>
 #include <vector>
 #include <unistd.h>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "rc.h"
 #include "screen.h"
 
@@ -42,7 +42,7 @@ const char * invocation_name = 0;
 const char * const Program_name     = "GNU Moe";
 const char * const program_name     = "moe";
 const char * const config_file_name = "moerc";
-const char * const program_year     = "2007";
+const char * const program_year     = "2008";
 
 
 void show_help() throw()
@@ -59,6 +59,8 @@ void show_help() throw()
   std::printf( "  -H, --smart-home         Go home, then go to first non-blank character\n" );
   std::printf( "  -i, --ignore-case        Make search case insensitive by default\n" );
   std::printf( "  -k, --keep-lines=<n>     Number of lines to keep for PgUp/PgDn\n" );
+  std::printf( "  -m, --max-windows=<n>    Max number of windows to show at once\n" );
+  std::printf( "  -n, --indent-step=<n>    Number of spaces to add when indenting a block\n" );
   std::printf( "  -1, --orphan             Put extra files in orphaned buffers\n" );
 //  std::printf( "  -p, --beep               Beep on errors and when cursor goes past extremes\n" );
   std::printf( "  -s, --search-wrap        Search wraps at end of file\n" );
@@ -128,7 +130,7 @@ bool use_it_anyway( std::string name ) throw()
 int process_config_file() throw()
   {
   std::fprintf( stderr, "\n" );
-  std::string name = RC::home_directory();
+  std::string name( RC::home_directory() );
   if( name.size() )
     {
     name += "/."; name += config_file_name;
@@ -165,7 +167,8 @@ void RC::show_error( const char * msg, const int errcode, const bool help ) thro
   if( msg && msg[0] != 0 )
     {
     std::fprintf( stderr, "%s: %s", program_name, msg );
-    if( errcode > 0 ) std::fprintf( stderr, ": %s", strerror( errcode ) );
+    if( errcode > 0 )
+      std::fprintf( stderr, ": %s", std::strerror( errcode ) );
     std::fprintf( stderr, "\n" );
     }
   if( help && invocation_name && invocation_name[0] != 0 )
@@ -173,24 +176,24 @@ void RC::show_error( const char * msg, const int errcode, const bool help ) thro
   }
 
 
-int main( int const argc, const char * argv[] ) throw()
+int main( const int argc, const char * argv[] ) throw()
   {
   invocation_name = argv[0];
 
-  for( int i = 1; i < argc; ++i )
+  int retval = RC::read_options( argc, argv );
+  if( retval )
     {
-    if( !std::strcmp( argv[i], "-h" ) || !std::strcmp( argv[i], "--help" ) )
-      { show_help(); return 0; }
-    if( !std::strcmp( argv[i], "-V" ) || !std::strcmp( argv[i], "--version" ) )
-      { show_version(); return 0; }
+    if( retval == 'h' ) { show_help(); return 0; }
+    if( retval == 'V' ) { show_version(); return 0; }
+    return retval;
     }
 
-//  setlocale( LC_ALL,"" );
+  Bufhandle_vector::init();
 
-  int retval = process_config_file();
+  retval = process_config_file();
   if( !retval )
     {
-    retval = RC::process_options( argc, argv );
+    retval = RC::process_options();
     if( retval == 2 )
       {
       std::fprintf( stderr, "Warning: non-regular files will be ignored.\n" );
