@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 */
 
 #include <algorithm>
+#include <cctype>
 #include <cerrno>
 #include <string>
 #include <vector>
@@ -23,6 +24,7 @@
 #include <sys/stat.h>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "iso_8859.h"
 #include "menu.h"
 #include "rc.h"
@@ -38,6 +40,9 @@ void show( const std::vector< std::string > & name_vector, const int lines,
   {
   static int top_line = 0;
   const int entries = name_vector.size();
+  const int bottom_line = ( entries - 1 ) / columns;
+  if( bottom_line - top_line + 1 < lines )
+    top_line = std::max( 0, bottom_line - lines + 1 );
   const int current_line = current / columns;
   if( current_line < top_line ) top_line = current_line;
   if( current_line >= top_line + lines ) top_line = current_line - lines + 1;
@@ -45,7 +50,7 @@ void show( const std::vector< std::string > & name_vector, const int lines,
 
   for( int line = 0; line < lines; ++line )
     {
-    int i1 = -1, i2 = 0, i_ini = first_entry + ( line * columns );
+    int i1 = -1, i2 = -1, i_ini = first_entry + ( line * columns );
     std::string s;
     for( int i = i_ini; i < i_ini + columns && i < entries; ++i )
       {
@@ -55,15 +60,28 @@ void show( const std::vector< std::string > & name_vector, const int lines,
         if( indicator ) ++i1;		// skip modified indicator
         s += name_vector[i];
         i2 = s.size();
-        cursor = Point( line, std::min( (int)s.size(), Screen::width() - 1 ) );
+        cursor = Point( line, std::min( i2, Screen::width() - 1 ) );
         }
       else s += name_vector[i];
       const unsigned int col = std::min( Screen::width(), ( ( ( i % columns ) + 1 ) * ( Screen::width() + 2 ) ) / columns );
       if( (int)col < Screen::width() && s.size() > col - 2 ) s.resize( col - 2 );
       while( s.size() < col ) s += ' ';
       }
-    if( i1 >= 0 ) { s.insert( i2, "@i" ); s.insert( i1, "@i" ); }
-    Screen::out_line( s, Screen::height() - lines + line - 1 );
+    Screen::out_menu_line( s, Screen::height() - lines + line - 1, i1, i2 );
+    }
+  }
+
+
+void append_common_prefix( std::string & name,
+                           const std::vector< std::string > & name_vector ) throw()
+  {
+  for( unsigned int i = 0; i < name_vector[0].size(); ++i )
+    {
+    const char ch = name_vector[0][i];
+    for( unsigned int j = 1; j < name_vector.size(); ++j )
+      if( i >= name_vector[j].size() || ch != name_vector[j][i] )
+        return;
+    name += ch;
     }
   }
 
@@ -129,11 +147,11 @@ int Menu::bufhandle_menu( int current, const int abort_key ) throw()
   {
   std::vector< std::string > name_vector;
 
-  const int entries = bufhandle_vector.handles();
+  const int entries = Bufhandle_vector::handles();
   for( int i = 0; i < entries; ++i )
     {
-    const int c = bufhandle_vector.circular_index( i );
-    const Buffer & buffer = bufhandle_vector[c].buffer();
+    const int c = Bufhandle_vector::circular_index( i );
+    const Buffer & buffer = Bufhandle_vector::handle( c ).buffer();
     std::string tmp( " " ); tmp += buffer.uname();
     if( buffer.modified() ) tmp[0] = '*';
     name_vector.push_back( tmp );
@@ -150,13 +168,14 @@ int Menu::bufhandle_menu( int current, const int abort_key ) throw()
   const int columns = std::max( 1, ( Screen::width() + 2 ) / ( max_width + 2 ) );
   const int _height = ( entries + columns - 1 ) / columns;
 
-  const int lines = std::min( 10, _height ), line = Screen::height() - lines - 1;
+  const int lines = std::min( 10, _height );
+  const int line = Screen::height() - lines - 1;
   Point cursor( 0, Screen::width() - 1 );
 
   Screen::save_lines_and_cursor( line, lines + 1 );
 
-  Screen::out_line( "Select buffer handle (^C to abort)", Screen::height() - 1 );
-  current = bufhandle_vector.absolute_index( current );
+  Screen::out_line( "Select buffer handle or 0-9 (^C to abort)", Screen::height() - 1 );
+  current = Bufhandle_vector::absolute_index( current );
   int key = 0;
   while( key >= 0 )
     {
@@ -168,7 +187,9 @@ int Menu::bufhandle_menu( int current, const int abort_key ) throw()
       valid_key = true;
       do key = Screen::wait_kbhit( line ); while( key < 0 );
       if( key == abort_key ) key = 3;
-      switch( key )
+      if( std::isdigit( key ) )
+        current = ( ( entries - 1 ) * ( key - '0' ) ) / 9;
+      else switch( key )
         {
         case 3   : key = -1; break;		// ^C
         case ' ' :
@@ -198,7 +219,11 @@ int Menu::bufhandle_menu( int current, const int abort_key ) throw()
     }
   Screen::restore_lines_and_cursor();
   if( key == -1 ) return -1;
-  else return bufhandle_vector.circular_index( current );
+  else
+    {
+    Bufhandle_vector::set_first( current );
+    return Bufhandle_vector::circular_index( current );	// should return 0
+    }
   }
 
 
@@ -220,10 +245,13 @@ bool Menu::file_menu( const std::string & prompt, std::string & name ) throw()
     {
     const struct dirent * entryp = readdir( dirp );
     if( !entryp ) { closedir( dirp ); break; }
-    std::string tmp = entryp->d_name;
+    std::string tmp( entryp->d_name );
     if( tmp != "." && Regex::match_filename( basename, tmp ) )
       name_vector.push_back( tmp );
     }
+  if( name_vector.size() > 1 )
+    { name = dirname; append_common_prefix( name, name_vector ); }
+
   const int entries = name_vector.size();
   if( !entries ) return true;
   if( entries > 1 ) std::sort( name_vector.begin(), name_vector.end() );
@@ -241,7 +269,8 @@ bool Menu::file_menu( const std::string & prompt, std::string & name ) throw()
   const int columns = std::max( 1, ( Screen::width() + 2 ) / ( max_width + 2 ) );
   const int _height = ( entries + columns - 1 ) / columns;
 
-  const int lines = std::min( 10, _height ), line = Screen::height() - lines - 1;
+  const int lines = std::min( 10, _height );
+  const int line = Screen::height() - lines - 1;
   Point cursor( 0, Screen::width() - 1 );
 
   Screen::save_lines_and_cursor( line, lines + 1 );
