@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 
 namespace Menu {
 
+const int columns = 3;
 int current;		// current highlighted entry in menu
 int entries;		// number of entries in menu
 int _height;		// height in lines of menu
@@ -54,6 +55,13 @@ bool beep_change( Buffer & ) throw()
   { RC::editor_options().beep = !RC::editor_options().beep; return true; }
 */
 
+const char * crlf_status( const Buffer & buffer ) throw()
+  { return buffer.crlf ? "ON" : "OFF"; }
+
+bool crlf_change( Buffer & buffer ) throw()
+  { buffer.crlf = !buffer.crlf; return true; }
+
+
 const char * exit_ask_status( const Buffer & ) throw()
   { return RC::editor_options().exit_ask ? "ON" : "OFF"; }
 
@@ -75,14 +83,37 @@ bool ignore_case_change( Buffer & ) throw()
   { RC::editor_options().ignore_case = !RC::editor_options().ignore_case; return true; }
 
 
-const char * lines_status( const Buffer & ) throw()
+const char * indent_step_status( const Buffer & ) throw()
+  {
+  static char buf[8];
+  snprintf( buf, sizeof( buf ), "%d", RC::editor_options().indent_step );
+  return buf;
+  }
+
+bool indent_step_change( Buffer & ) throw()
+  {
+  static std::vector< std::string > history;
+
+  history.push_back( std::string() );
+  if( Screen::get_string( "Spaces to add when indenting a block (^C to abort): ", history ) <= 0 )
+    return false;
+  std::string & s = history.back();
+  int i;
+  if( !RC::parse_int( s, i ) || i < 1 || i >= Screen::width() )
+    { Screen::show_message( "Value out of range" ); history.pop_back();
+    return false; }
+  RC::editor_options().indent_step = i; return true;
+  }
+
+
+const char * keep_lines_status( const Buffer & ) throw()
   {
   static char buf[8];
   snprintf( buf, sizeof( buf ), "%d", RC::editor_options().keep_lines );
   return buf;
   }
 
-bool lines_change( Buffer & ) throw()
+bool keep_lines_change( Buffer & ) throw()
   {
   static std::vector< std::string > history;
 
@@ -121,6 +152,29 @@ bool lmargin_change( Buffer & buffer ) throw()
   }
 
 
+const char * max_windows_status( const Buffer & ) throw()
+  {
+  static char buf[8];
+  snprintf( buf, sizeof( buf ), "%d", RC::editor_options().max_windows );
+  return buf;
+  }
+
+bool max_windows_change( Buffer & ) throw()
+  {
+  static std::vector< std::string > history;
+
+  history.push_back( std::string() );
+  if( Screen::get_string( "Max number of windows to show or 0 for no limit (^C to abort): ", history ) <= 0 )
+    return false;
+  std::string & s = history.back();
+  int i;
+  if( !RC::parse_int( s, i ) || i < 0 || i > Screen::max_windows() )
+    { Screen::show_message( "Value out of range" ); history.pop_back();
+    return false; }
+  RC::editor_options().max_windows = i; return true;
+  }
+
+
 const char * read_only_status( const Buffer & buffer ) throw()
   { return buffer.options.read_only ? "ON" : "OFF"; }
 
@@ -195,11 +249,14 @@ struct Entry
 Entry table[] = {
   { 'A', "Autoindent ", &autoindent_status, &autoindent_change },
   { 'B', "Create backups ", &backup_status, &backup_change },
+  { 'C', "CR-LF ended lines ", &crlf_status, &crlf_change },
   { 'E', "Exit ask ", &exit_ask_status, &exit_ask_change },
   { 'H', "Smart home key ", &smart_home_status, &smart_home_change },
   { 'I', "Search ignores case ", &ignore_case_status, &ignore_case_change },
-  { 'K', "Keep PgUp/PgDn lines ", &lines_status, &lines_change },
+  { 'K', "Keep PgUp/PgDn lines ", &keep_lines_status, &keep_lines_change },
   { 'L', "Left margin ", &lmargin_status, &lmargin_change },
+  { 'M', "Max windows to show ", &max_windows_status, &max_windows_change },
+  { 'N', "Indent step ", &indent_step_status, &indent_step_change },
   { 'O', "Read only ", &read_only_status, &read_only_change },
 //  { 'P', "Beeps ", &beep_status, &beep_change },
   { 'R', "Right margin ", &rmargin_status, &rmargin_change },
@@ -217,7 +274,7 @@ struct Init_menu
     {
     current = 0;
     for( entries = 0; table[entries].ch; ++entries ) ;
-    _height = ( entries + 2 ) / 3;
+    _height = ( entries + columns - 1 ) / columns;
     }
   } dummy;
 
@@ -235,24 +292,20 @@ bool set_current( int key ) throw()
   }
 
 
-void show( const int delta, const Buffer & buffer, Point & cursor ) throw()
+void show( const Buffer & buffer, Point & cursor ) throw()
   {
-  if( delta > 0 ) { if( current + delta >= entries ) return; }
-  else if( delta < 0 ) { if( current + delta < 0 ) return; }
-  current += delta;
-
   for( int line = 0; line < _height; ++line )
     {
     int offset = 0;
     std::string s;
-    for( int i = 3 * line; i < 3 * ( line + 1 ) && i < entries; ++i )
+    for( int i = columns * line; i < columns * ( line + 1 ) && i < entries; ++i )
       {
       if( i == current )
         { cursor = Point( line, s.size() ); s += "@i"; offset += 2; }
       s += table[i].ch; s += ' '; s += table[i].str;
       if( table[i].status )
         { const char * st = table[i].status( buffer ); if( st ) s += st; }
-      unsigned int col = ( ( ( i % 3 ) + 1 ) * Screen::width() ) / 3 + offset;
+      unsigned int col = ( ( ( i % columns ) + 1 ) * Screen::width() ) / columns + offset;
       while( s.size() < col ) s += ' ';
       if( i == current ) { s += "@i"; offset += 2; }
       }
@@ -265,16 +318,17 @@ void show( const int delta, const Buffer & buffer, Point & cursor ) throw()
 
 void Menu::options_menu( Buffer & buffer, const int abort_key ) throw()
   {
-  const int lines = _height, line = Screen::height() - lines;
+  const int lines = _height;
+  const int line = Screen::height() - lines;
   Point cursor( Screen::height() - 1, Screen::width() - 1 );
 
   Screen::save_lines_and_cursor( line, lines );
 
-  int key = abort_key + 1, delta = 0;
+  int key = abort_key + 1;
   bool valid_command = false;
   while( key != abort_key && !valid_command )
     {
-    show( delta, buffer, cursor );
+    show( buffer, cursor );
     Screen::move_to( Point( line, 0 ) + cursor );
     bool valid_key = false;
     while( !valid_key )
@@ -289,10 +343,18 @@ void Menu::options_menu( Buffer & buffer, const int abort_key ) throw()
         default: if( set_current( key ) ) valid_command = true;
                  else switch( Screen::convert_key( key ) )
                    {
-                   case Screen::key_up   : delta = -3; break;
-                   case Screen::key_down : delta = +3; break;
-                   case Screen::key_left : delta = -1; break;
-                   case Screen::key_right: delta = +1; break;
+                   case Screen::key_up   :
+                     if( current >= columns ) current -= columns; break;
+                   case Screen::key_down :
+                     if( current + columns < entries ) current += columns; break;
+                   case Screen::key_left : if( current > 0 ) --current; break;
+                   case Screen::key_right: if( current < entries - 1 ) ++current; break;
+                   case Screen::key_ppage:
+                     while( current >= columns ) current -= columns; break;
+                   case Screen::key_npage:
+                     while( current + columns < entries ) current += columns; break;
+                   case Screen::key_home : current = 0; break;
+                   case Screen::key_end  : current = entries - 1; break;
                    default: valid_key = false;
                    }
         }
@@ -304,7 +366,7 @@ void Menu::options_menu( Buffer & buffer, const int abort_key ) throw()
     const Entry & entry = table[current];
     if( entry.change && entry.change( buffer ) )
       {
-      std::string msg = entry.str;
+      std::string msg( entry.str );
       if( entry.status )
         { const char * st = entry.status( buffer ); if( st ) msg += st; }
       if( msg.size() )
