@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,12 +17,14 @@
 
 #include <cctype>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
 #include <string>
 #include <vector>
 
 #include "arg_parser.h"
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "menu.h"
 #include "rc.h"
 #include "regex.h"
@@ -30,6 +32,8 @@
 
 namespace RC {
 
+const Arg_parser * parserp;
+
 // priority of _default_buffer_options by place of specification:
 // ( 0 = lowest, 3 = highest )
 // 0 default priority
@@ -66,9 +70,10 @@ std::vector< Regex_options > regex_options_vector;
 enum Optcode
   {
   auto_unmark = 'u', backup = 'b', /*beep = 'p',*/ exit_ask = 'e',
-  ignore_case = 'i', keep_lines = 'k', orphan_extra= '1', rectangle_mode = 'x',
-  search_wrap = 's', smart_home = 'H', auto_indent = 'a', lmargin = 'l',
-  rmargin = 'r', overwrite = 'O', read_only = 'o', word_wrap = 'w',
+  ignore_case = 'i', indent_step = 'n', keep_lines = 'k', max_windows = 'm',
+  orphan_extra= '1', rectangle_mode = 'x', search_wrap = 's', smart_home = 'H',
+  auto_indent = 'a', lmargin = 'l', rmargin = 'r', overwrite = 'O',
+  read_only = 'o', word_wrap = 'w',
   no_auto_unmark = 256, no_backup, /*no_beep,*/ no_exit_ask, no_ignore_case,
   no_rectangle_mode, no_search_wrap, no_smart_home, no_auto_indent,
   no_overwrite, no_read_only, no_word_wrap,
@@ -76,6 +81,8 @@ enum Optcode
 
 const Arg_parser::Option options[] =
   {
+  { 'h',               "help",           Arg_parser::no  },
+  { 'V',               "version",        Arg_parser::no  },
   { auto_unmark,       "auto-unmark",    Arg_parser::no  },
   { no_auto_unmark,    "no-auto-unmark", Arg_parser::no  },
   { backup,            "backup",         Arg_parser::no  },
@@ -86,7 +93,9 @@ const Arg_parser::Option options[] =
   { no_exit_ask,       "no-exit-ask",    Arg_parser::no  },
   { ignore_case,       "ignore-case",    Arg_parser::no  },
   { no_ignore_case,    "no-ignore-case", Arg_parser::no  },
+  { indent_step,       "indent-step",    Arg_parser::yes },
   { keep_lines,        "keep-lines",     Arg_parser::yes },
+  { max_windows,       "max-windows",    Arg_parser::yes },
   { orphan_extra,      "orphan",         Arg_parser::no  },
   { rectangle_mode,    "rectangle",      Arg_parser::no  },
   { no_rectangle_mode, "no-rectangle",   Arg_parser::no  },
@@ -164,11 +173,11 @@ bool set_int_option( int & option, const std::string & arg,
 
 bool set_option_from_code( const int code, const std::string & arg,
                            Options *edop, Buffer::Options *bufop,
-                           int priority = 0, char *msg = "" )
+                           int priority = 0, const char * msg = "" )
   {
   if( ( is_editor_option( code ) && !edop ) ||
       ( is_buffer_option( code ) && !bufop ) )
-    { std::fprintf( stderr, "%soption `%s' not allowed here\n", msg, optname( code ) );
+    { std::fprintf( stderr, "%serror: option `%s' not allowed here\n", msg, optname( code ) );
       return false; }
 
   int errcode = 0;
@@ -184,8 +193,12 @@ bool set_option_from_code( const int code, const std::string & arg,
     case no_exit_ask:       edop->exit_ask = false; break;
     case ignore_case:       edop->ignore_case = true; break;
     case no_ignore_case:    edop->ignore_case = false; break;
+    case indent_step:       if( !parse_int( arg, edop->indent_step ) ) errcode = 1;
+                            break;
     case keep_lines:        if( !parse_int( arg, edop->keep_lines ) ) errcode = 1;
                             break;
+    case max_windows:       if( !parse_int( arg, edop->max_windows ) ) errcode = 1;
+                            break;
     case orphan_extra:      edop->orphan_extra = true; break;
     case rectangle_mode:    edop->rectangle_mode = true; break;
     case no_rectangle_mode: edop->rectangle_mode = false; break;
@@ -227,9 +240,9 @@ bool set_option_from_code( const int code, const std::string & arg,
 
   if( !errcode ) return true;
   if( errcode == 1 )
-    std::fprintf( stderr, "%sbad argument for option `%s'\n", msg, optname( code ) );
+    std::fprintf( stderr, "%serror: bad argument for option `%s'\n", msg, optname( code ) );
   else if( errcode == 2 )
-    std::fprintf( stderr, "%svalue out of range for option `%s'\n", msg, optname( code ) );
+    std::fprintf( stderr, "%serror: value out of range for option `%s'\n", msg, optname( code ) );
   return false;
   }
 
@@ -275,7 +288,7 @@ bool RC::parse_int( const std::string & s, int & result ) throw()
   if( s.size() == 0 ) return false;
   const char *str = s.c_str();
   char *tail;
-  result = strtol( str, &tail, 0 );
+  result = std::strtol( str, &tail, 0 );
   return ( tail != str );
   }
 
@@ -302,7 +315,7 @@ int RC::process_rcfile( const std::string & name ) throw()
     if( len <= 1 || std::isspace( buf[0] ) ) continue;
     if( buf[0] != '-' )		// Create new file name dependent options
       {
-      std::string regex = strtok( buf, delimiters );
+      std::string regex( std::strtok( buf, delimiters ) );
       if( regex_status != 1 )
         regex_options_vector.push_back( Regex_options() );
       regex_options_vector.back().regex_vector.push_back( regex );
@@ -310,8 +323,8 @@ int RC::process_rcfile( const std::string & name ) throw()
       }
     else			// Set an option
       {
-      char *opt = strtok( buf, delimiters );
-      char *arg = strtok( 0, delimiters );
+      char *opt = std::strtok( buf, delimiters );
+      char *arg = std::strtok( 0, delimiters );
       char msg[80];
       snprintf( msg, sizeof(msg), "\n%s %d: ", name.c_str(), line );
       Arg_parser parser( opt, arg, options );
@@ -338,20 +351,17 @@ int RC::process_rcfile( const std::string & name ) throw()
 
 
     // Returns 0 for success, 1 for invalid option, 2 if non-regular files given.
-int RC::process_options( const int argc, const char * const argv[] ) throw()
+int RC::process_options() throw()
   {
   std::fprintf( stderr, "Processing options... " );
   std::fflush( stderr );
 
-  Arg_parser parser( argc, argv, options, true );
-  if( parser.error().size() )				// bad option
-    { show_error( parser.error().c_str(), 0, true ); return 1; }
-
   int argind, line = 0;
-  for( argind = 0; argind < parser.arguments(); ++argind )
+  bool first_post = true;
+  for( argind = 0; argind < parserp->arguments(); ++argind )
     {						// Process "global" options
-    const int code = parser.code( argind );
-    const std::string & arg = parser.argument( argind );
+    const int code = parserp->code( argind );
+    const std::string & arg = parserp->argument( argind );
     if( !code )
       {
       if( arg[0] != '+' ) break;			// filename
@@ -359,51 +369,75 @@ int RC::process_options( const int argc, const char * const argv[] ) throw()
       show_error( "bad line number" ); return 1;
       }
     if( !set_option_from_code( code, arg, &_editor_options,
-                               &_default_buffer_options, 3 ) ) return 1;
+                               &_default_buffer_options, 3,
+                               first_post ? "\n" : "" ) )
+      { first_post = false; return 1; }
     }
 
   bool non_regular_found = false;
-  try {					// Process input files and file options
-    while( argind < parser.arguments() )
+  try {
+    while( argind < parserp->arguments() )  // Process files and file options
       {
-      std::string name = parser.argument( argind++ );
+      Buffer * bufferp = 0;
+      std::string name( parserp->argument( argind++ ) );
       Menu::tilde_expansion( name );
-      const bool regular = Menu::is_regular( name, true );
-      if( !regular )
+      if( Menu::is_regular( name, true ) )
         {
-        std::fprintf( stderr, "warning: `%s' is not a regular file\n", name.c_str() );
-        name.clear(); non_regular_found = true;
+        const int i = Bufhandle_vector::find_or_add_handle( _default_buffer_options, &name, -1, line );
+        bufferp = &Bufhandle_vector::handle( i ).buffer();
         }
-      const int i = bufhandle_vector.find_or_add_handle( _default_buffer_options, &name, -1, line );
-      line = 0;
-      Buffer & buffer = bufhandle_vector[i].buffer();
-      for( ; argind < parser.arguments(); ++argind )	// Process file options
+      else
         {
-        const int code = parser.code( argind );
-        const std::string & arg = parser.argument( argind );
+        if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
+        std::fprintf( stderr, "warning: `%s' is not a regular file\n", name.c_str() );
+        non_regular_found = true;
+        }
+      line = 0;
+      for( ; argind < parserp->arguments(); ++argind )	// Process file options
+        {
+        const int code = parserp->code( argind );
+        const std::string & arg = parserp->argument( argind );
         if( !code )
           {
           if( arg[0] != '+' ) break;			// filename
           if( parse_int( arg, line ) && --line >= 0 ) continue;
+          if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
           show_error( "bad line number" ); return 1;
           }
-        if( !set_option_from_code( code, parser.argument( argind ),
-                                   0, &buffer.options ) ) return 1;
+        if( bufferp &&
+            !set_option_from_code( code, parserp->argument( argind ),
+                                   0, &bufferp->options, 0,
+                                   first_post ? "\n" : "" ) )
+          { first_post = false; return 1; }
         }
-      if( !regular ) bufhandle_vector.delete_handle( i );
       }
+    // Add empty buffer if no files given or stdin is not a tty
+    Bufhandle_vector::add_handle_if_pending_input_or_empty( line );
+    }
+  catch( Buffer::Error e )
+    {
+    if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
+    show_error( e.s ); return 1;
     }
-  catch( Buffer::Error e ) { show_error( e.s ); return 1; }
-
-  // Add empty buffer if no files given or stdin is not a tty
-  try { bufhandle_vector.add_handle_if_pending_input_or_empty( line ); }
-  catch( Buffer::Error e ) { show_error( e.s ); return 1; }
 
   std::fprintf( stderr, "done\n" );
+  delete parserp;
   if( non_regular_found ) return 2; else return 0;
   }
 
 
+int RC::read_options( const int argc, const char * const argv[] ) throw()
+  {
+  parserp = new Arg_parser( argc, argv, options, true );
+  if( parserp->error().size() )				// bad option
+    { show_error( parserp->error().c_str(), 0, true ); return 1; }
+  for( int i = 0; i < parserp->arguments(); ++i )
+    if( parserp->code( i ) == 'h' || parserp->code( i ) == 'V' )
+      return parserp->code( i );
+  return 0;
+  }
+
+
 void RC::reset() throw()
   {
   _editor_options.reset();
