@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,15 +19,15 @@ namespace RC {
 
 struct Options
   {
-  int keep_lines;
+  int indent_step, keep_lines, max_windows;
   bool auto_unmark, backup, /*beep,*/ exit_ask, ignore_case;
-  bool orphan_extra, rectangle_mode, smart_home, search_wrap;
+  bool orphan_extra, rectangle_mode, search_wrap, show_code, smart_home;
 
   Options() throw()
-    : keep_lines( -1 ),
+    : indent_step( 2 ), keep_lines( -1 ), max_windows( 0 ),
     auto_unmark( false ), backup( true ), /*beep( false ),*/ exit_ask( false ),
     ignore_case( false ), orphan_extra( false ), rectangle_mode( false ),
-    smart_home( true ), search_wrap( false ) {}
+    search_wrap( false ), show_code( false ), smart_home( true ) {}
 
   void reset() throw() { *this = Options(); }
   };
@@ -44,7 +44,9 @@ bool parse_int( const std::string & s, int & result ) throw();
 int process_rcfile( const std::string & name ) throw();
 
     // Returns 0 for success, 1 for invalid option, 2 for a no regular file.
-int process_options( const int argc, const char * const argv[] ) throw();
+int process_options() throw();
+
+int read_options( const int argc, const char * const argv[] ) throw();
 
 void reset() throw();
 
