@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@ namespace Regex {
 // found substring and returns in 'pieces' the text matching every \*,
 // \? etc from the regular expresion.
 //
-bool find( const Buffer & buffer, Point & p1, Point & p2,
+bool find( const Basic_buffer & buffer, Point & p1, Point & p2,
            const std::string & regex, std::vector< std::string > & pieces,
-           bool icase = false, bool backward = false ) throw();
+           const bool icase = false, const bool backward = false ) throw();
 
 
 // Deletes block at [p1,p2) and replaces it with the text produced from
@@ -39,6 +39,7 @@ bool replace( Buffer & buffer, const Point & p1, Point & p2,
 
 // returns true if 'name' matches 'regex'
 //
-bool match_filename( const std::string & regex, const std::string & name ) throw();
+bool match_filename( const std::string & regex, const std::string & name,
+                     unsigned int i1 = 0, unsigned int i2 = 0 ) throw();
 
 } // end namespace Regex
