@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,12 +19,14 @@
 #include <cctype>
 #include <cerrno>
 #include <cstdlib>
+#include <ctime>
 #include <string>
 #include <vector>
 #include <unistd.h>
 #include <ncurses.h>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "block.h"
 #include "iso_8859.h"
 #include "menu.h"
@@ -50,6 +52,7 @@ int _height, _width;
 int clock_limit;		//   0: disable clock
 				//   1: showing help
 				// > 1: first line used by menu
+std::string _clock_string( " 0:00" );
 
 std::vector< Lines_and_cursor > lines_and_cursor_vector;
 
@@ -180,7 +183,7 @@ void control_k_loop( int key ) throw()
   }
 
 
-void control_o_loop( int key, const std::string & version_msg ) throw()
+void control_o_loop( int key ) throw()
   {
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
@@ -196,9 +199,12 @@ void control_o_loop( int key, const std::string & version_msg ) throw()
     case '6': Window_vector::decode_quoted_printable_utf8( true ); break;
     case '8': Window_vector::decode_quoted_printable_utf8( false ); break;
     case 'C': Window_vector::center_line(); break;
+    case 'D': Window_vector::remove_duplicate_lines(); break;
+    case 'E': Window_vector::remove_duplicate_lines( true ); break;
+    case 'L': Window_vector::change_case( false ); break;
     case 'R': werase( stdscr ); wrefresh( stdscr ); repaint(); break;
     case 'S': Window_vector::split_window(); break;
-    case 'V': show_message( version_msg ); break;
+    case 'U': Window_vector::change_case( true ); break;
     }
   }
 
@@ -212,17 +218,35 @@ void control_q_loop( int key, int & retval ) throw()
   key = ISO_8859::decontrolize( key );
   switch( key )
     {
-    case KEY_F(2): bufhandle_vector.save_all_named(); break;
+    case KEY_F(2): Bufhandle_vector::save_all_named(); break;
     case 'C': if( Window_vector::close_and_exit( true ) == 0 ) retval = 0; break;
-    case 'S': bufhandle_vector.show_status(); break;
+    case 'U': Window_vector::copyright_update(); break;
     case 'X': if( Window_vector::close_and_exit( false ) == 0 ) retval = 0; break;
     }
   }
 
 
+void control_s_loop( int key, const std::string & version_msg ) throw()
+  {
+  Window & w = Window_vector::curwin();
+  w.show_status_line( ISO_8859::control_name( key ) );
+  do key = wait_kbhit(); while( key < 0 );
+  w.show_status_line();
+  key = ISO_8859::decontrolize( key );
+  switch( key )
+    {
+    case 'C': RC::editor_options().show_code = !RC::editor_options().show_code;
+              show_status_lines(); break;
+    case 'G': Bufhandle_vector::show_status(); break;
+    case 'I': w.show_character_info(); break;
+    case 'V': show_message( version_msg ); break;
+    }
+  }
+
+
 void user_control_char() throw()
   {
-  const std::string prompt = "Ctrl(A-Z) or 0-255 or 0x00-0xFF or 0-0377 or Escape sequence: ";
+  const std::string prompt( "Ctrl(A-Z) or 0-255 or 0x00-0xFF or 0-0377 or Escape sequence: " );
   int key;
   while( true )
     {
@@ -286,6 +310,7 @@ bool Screen::init() throw()
   getmaxyx( stdscr, _height, _width );
   if( _height < 24 || _width < 80 ) return false;
   clock_limit = 0;
+  clock_handler( 0 );
   Window_vector::init();
   return true;
   }
@@ -294,28 +319,34 @@ bool Screen::init() throw()
 //void Screen::beep() throw() { ::beep(); wrefresh( stdscr ); }
 
 
+const std::string & Screen::clock_string() throw() { return _clock_string; }
+
+
 void Screen::clock_handler( const int sig ) throw()	// updates the clock
   {
-  static std::string s( " 0:00" );
-
   time_t dummy = time( 0 ); struct tm *t = localtime( &dummy );
   if( t )
     {
     int h = t->tm_hour % 12; if( h == 0 ) h = 12;
-    s[0] = ( h >= 10 ) ? '0' + ( h / 10 ) : ' '; s[1] = '0' + ( h % 10 );
-    s[2] = ( sig == 0 || s[2] != ':') ? ':' : ' ';
-    s[3] = '0' + ( t->tm_min / 10 ); s[4] = '0' + ( t->tm_min % 10 );
-    Point old_cursor; save_cursor( old_cursor );
-    if( clock_limit == 1 )
-      out_raw_string( 0, 60, s, 5, 0, A_REVERSE );
-    else for( int i = 0; i < Window_vector::windows(); ++i )
+    _clock_string[0] = ( h >= 10 ) ? '0' + ( h / 10 ) : ' ';
+    _clock_string[1] = '0' + ( h % 10 );
+    _clock_string[2] = (_clock_string[2] != ':') ? ':' : ' ';
+    _clock_string[3] = '0' + ( t->tm_min / 10 );
+    _clock_string[4] = '0' + ( t->tm_min % 10 );
+    if( sig )
       {
-      Point cursor = Window_vector::window( i ).clock_position();
-      if( cursor.line >= clock_limit ) break;
-      out_raw_string( cursor.line, cursor.col, s, 5, 0, A_REVERSE );
+      Point old_cursor; save_cursor( old_cursor );
+      if( clock_limit == 1 )
+        out_raw_string( 0, 60, _clock_string, 5, 0, A_REVERSE );
+      else for( int i = 0; i < Window_vector::windows(); ++i )
+        {
+        const Point cursor = Window_vector::window( i ).clock_position();
+        if( cursor.line >= clock_limit ) break;
+        out_raw_string( cursor.line, cursor.col, _clock_string, 5, 0, A_REVERSE );
+        }
+      wmove( stdscr, old_cursor.line, old_cursor.col );
+      wrefresh( stdscr ); alarm( 1 );
       }
-    wmove( stdscr, old_cursor.line, old_cursor.col );
-    if( sig ) { wrefresh( stdscr ); alarm( 1 ); }
     }
   }
 
@@ -337,7 +368,7 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
       move_to( Window_vector::curwin().absolute_cursor() );
 
     clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
-    if( clock_limit > 0 ) { clock_handler( 0 ); alarm( 1 ); }
+    if( clock_limit > 0 ) alarm( 1 );
     wrefresh( stdscr );
     }
 
@@ -360,7 +391,7 @@ int Screen::get_string( const std::string & prompt,
   const int line = _height - 1, lines = 1;
   save_lines_and_cursor( line, lines );
 
-  const std::string original = history.back();
+  const std::string original( history.back() );
   remove_duplicates( history );
   std::string s( prompt );
   s += history.back();
@@ -379,10 +410,15 @@ int Screen::get_string( const std::string & prompt,
       case   3 : key = -1; break;				// ^C
       case '\t': if( usetab )
                    {
-                   std::string name( s, prompt.size(), s.size() );
+                   const int namelen = scol - prompt.size();
+                   std::string name( s, prompt.size(), namelen );
                    if( !Menu::file_menu( prompt, name ) ) { key = -1; break; }
-                   s.erase( prompt.size(), s.size() ); s += name;
-                   scol = s.size(); modified = true;
+                   scol += name.size() - namelen; modified = true;
+                   s.erase( prompt.size(), namelen );
+                   if( s.size() > prompt.size() && name.size() &&
+                       s[prompt.size()] == '/' && name[name.size()-1] == '/' )
+                     name.erase( name.size() - 1 );
+                   s.insert( prompt.size(), name );
                    }
                  break;
       case '\r': {
@@ -417,6 +453,7 @@ int Screen::get_string( const std::string & prompt,
       case KEY_END  : scol = s.size(); break;
       case KEY_LEFT : if( scol > prompt.size() ) { --scol; } break;
       case KEY_RIGHT: if( scol < s.size() ) { ++scol; } break;
+      case KEY_PPAGE:
       case KEY_UP   : if( idx > 0 )
         {
         if( modified )
@@ -427,8 +464,10 @@ int Screen::get_string( const std::string & prompt,
           else if( idx == history.size() - 1 ) history[idx].clear();
           else history.erase( history.begin() + idx );
           }
-        s.erase( prompt.size(), s.size() ); s += history[--idx]; scol = s.size();
+        if( key == KEY_UP ) --idx; else idx = 0;
+        s.erase( prompt.size(), s.size() ); s += history[idx]; scol = s.size();
         } break;
+      case KEY_NPAGE:
       case KEY_DOWN : if( idx + 1 < history.size() )
         {
         if( modified )
@@ -438,7 +477,8 @@ int Screen::get_string( const std::string & prompt,
             history[idx].assign( s, prompt.size(), s.size() );
           else { history.erase( history.begin() + idx ); --idx; }
           }
-        s.erase( prompt.size(), s.size() ); s += history[++idx]; scol = s.size();
+        if( key == KEY_DOWN ) ++idx; else idx = history.size() - 1;
+        s.erase( prompt.size(), s.size() ); s += history[idx]; scol = s.size();
         } break;
       default: if( key < 256 && ISO_8859::isprint( key ) )
                  { s.insert( scol++, 1, key ); modified = true; }
@@ -460,6 +500,8 @@ int Screen::height() throw() { return _height; }
 
 int Screen::width() throw() { return _width; }
 
+int Screen::max_windows() throw() { return _height / Window::min_height; }
+
 
 void Screen::move_to( const Point & cursor ) throw()
   { wmove( stdscr, cursor.line, cursor.col ); }
@@ -540,6 +582,24 @@ bool Screen::out_line( const std::string & s, const int line, const bool shift,
   }
 
 
+bool Screen::out_menu_line( const std::string & s, const int line,
+                            const int hbegin, const int hend ) throw()
+  {
+  if( line < 0 || line >= _height ) return false;
+  if( hbegin >= 0 && hbegin < hend && hend <= (int)s.size() )
+    {
+    if( hbegin > 0 )
+      out_raw_string( line, 0, s, hbegin, 0, A_NORMAL );
+    if( hbegin < _width )
+      out_raw_string( line, hbegin, s, hend - hbegin, hbegin, A_REVERSE );
+    if( hend < _width )
+      out_raw_string( line, hend, s, _width, hend, A_NORMAL );
+    }
+  else out_raw_string( line, 0, s, _width, 0, A_NORMAL );
+  return true;
+  }
+
+
 void Screen::repaint( const Buffer * bufp ) throw()
   {
   for( int i = 0; i < Window_vector::windows(); ++i )
@@ -615,10 +675,11 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case  6: Window_vector::search( key ); break;		// ^F
       case  7: Window_vector::search( key, true ); break;	// ^G
       case 11: control_k_loop( key ); break;			// ^K
-      case 15: control_o_loop( key, version_msg ); break;	// ^O
+      case 15: control_o_loop( key ); break;			// ^O
       case 16: user_control_char(); break;			// ^P
       case 17: control_q_loop( key, retval ); break;		// ^Q
-      case 19: w.show_character_info(); break;			// ^S
+      case 19: control_s_loop( key, version_msg ); break;	// ^S
+      case 23: Window_vector::search_word( key ); break;	// ^W
       case 24: if( !Window_vector::close( false ) ) retval = 0; break;	// ^X
       case 25: Window_vector::delete_line(); break;		// ^Y
       case 27: alt_key_loop( key ); break;			// Alt or Esc
