@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@ bool init() throw();
 
 //void beep() throw();
 
+const std::string & clock_string() throw();
+
 void clock_handler( const int sig ) throw();		// Updates the clock.
 
 int kbhit() throw();			// Returns -1 if no key is available.
@@ -49,6 +51,7 @@ int get_string( const std::string & prompt,
 
 int height() throw();
 int width() throw();
+int max_windows() throw();
 
 void move_to( const Point & cursor ) throw();
 void save_cursor( Point & cursor ) throw();
@@ -58,6 +61,8 @@ void restore_lines_and_cursor() throw();
 void out_buffer_line( const Buffer & buffer, Point c, const int line ) throw();
 bool out_line( const std::string & s, const int line, const bool shift = false,
                const char mode = 0 ) throw();
+bool out_menu_line( const std::string & s, const int line,
+                    const int hbegin = -1, const int hend = -1 ) throw();
 void repaint( const Buffer * bufp = 0 ) throw();
 int show_message( const std::string & s, const bool get_key = false,
                   const bool take_cursor = false ) throw();
