@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,12 @@
 */
 
 #include <algorithm>
+#include <cstring>
 #include <string>
 #include <vector>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "block.h"
 #include "iso_8859.h"
 #include "menu.h"
@@ -323,7 +325,7 @@ void Window::show_status_line( const char * prefix ) const throw()
   const int overlap = buffer().uname().size() + 11 - offset;
   if( overlap > 0 ) uname += overlap;
   const bool modified = buffer().modified();
-  const int len = snprintf( buf, sizeof( buf ), "%-3s %c%c%c%c%c%c %s%s",
+  const int len = snprintf( buf, offset + 1, "%-3s %c%c%c%c%c%c %s%s",
                             prefix ? prefix : "   ",
                             buffer().options.overwrite ? 'O' : 'I',
                             buffer().options.word_wrap ? 'W' : ' ',
@@ -334,7 +336,17 @@ void Window::show_status_line( const char * prefix ) const throw()
                             modified ? " (Modified)" : "" );
   if( len < offset ) std::memset( buf + len, ' ', offset - len );
   snprintf( buf + offset, sizeof( buf ) - offset,
-            "  Line %-4d Col %-3d   :    F1 for help", line, col );
+            "  Line %-4d Col %-3d %5s  F1 for help",
+            line, col, Screen::clock_string().c_str() );
+  if( RC::editor_options().show_code )
+    {
+    offset += 27;
+    const int ch = bhp->buffer()[bhp->pointer];
+    if( ch >= 0 )
+      snprintf( buf + offset, sizeof( buf ) - offset, "%3d(0x%02X)", ch, ch );
+    else
+      snprintf( buf + offset, sizeof( buf ) - offset, "End of file" );
+    }
   Screen::out_line( buf, _top_line, false, 'i' );
   }
 
