@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,10 +17,12 @@
 
 #include <algorithm>
 #include <cctype>
+#include <ctime>
 #include <string>
 #include <vector>
 
 #include "buffer.h"
+#include "buffer_handle.h"
 #include "block.h"
 #include "encoding.h"
 #include "iso_8859.h"
@@ -89,7 +91,7 @@ void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp ) throw()
 int Window_vector::init() throw()
   {
   _curwin = 0;
-  return load( RC::editor_options().orphan_extra ? 1 : bufhandle_vector.handles() );
+  return load( RC::editor_options().orphan_extra ? 1 : Bufhandle_vector::handles() );
   }
 
 
@@ -98,7 +100,7 @@ int Window_vector::init() throw()
 //
 int Window_vector::load( int new_size ) throw()
   {
-  const int handles = bufhandle_vector.handles();
+  const int handles = Bufhandle_vector::handles();
   const bool toggle = ( new_size < 0 && handles > 1 );
   const bool center = ( data.size() == 0 );		// center all on init
 
@@ -106,24 +108,26 @@ int Window_vector::load( int new_size ) throw()
   if( new_size == 0 ) new_size = data.size();
   else if( new_size < 0 )
     {
-    for( int i = 0; i < _curwin; ++i ) bufhandle_vector.next();
+    for( int i = 0; i < _curwin; ++i ) Bufhandle_vector::next();
     new_size = (data.size() != 1) ? 1 : handles;
     _curwin = 0;
     }
   new_size = std::min( new_size, handles );
-  new_size = std::min( new_size, Screen::height() / Window::min_height );
+  new_size = std::min( new_size, Screen::max_windows() );
+  if( RC::editor_options().max_windows > 0 )
+    new_size = std::min( new_size, RC::editor_options().max_windows );
   new_size = std::max( new_size, 1 );
-  while( _curwin >= new_size ) { --_curwin; bufhandle_vector.next(); }
+  while( _curwin >= new_size ) { --_curwin; Bufhandle_vector::next(); }
   const int wheight = Screen::height() / new_size;
 
   data.clear();
   int i = 0, y = 0;
   while( i < new_size - 1 )
     {
-    data.push_back( Window( y, wheight, bufhandle_vector[i], center ) );
+    data.push_back( Window( y, wheight, Bufhandle_vector::handle( i ), center ) );
     ++i; y += wheight;
     }
-  data.push_back( Window( y, Screen::height() - y, bufhandle_vector[i], center ) );
+  data.push_back( Window( y, Screen::height() - y, Bufhandle_vector::handle( i ), center ) );
   if( toggle && new_size == 1 ) data[0].center_cursor();
   for( i = 0; i < new_size; ++i ) data[i].repaint();
   return data.size();
@@ -142,13 +146,13 @@ void Window_vector::load_file( const int abort_key ) throw()
   std::string name;
   if( Menu::get_filename( "Name of file to edit (^C to abort): ", name, abort_key ) < 0 )
     return;
-  const int old_handles = bufhandle_vector.handles();
+  const int old_handles = Bufhandle_vector::handles();
   int handle;
-  try { handle = bufhandle_vector.find_or_add_handle( RC::default_buffer_options(),
+  try { handle = Bufhandle_vector::find_or_add_handle( RC::default_buffer_options(),
                                                       &name, _curwin + 1 ); }
   catch( Buffer::Error e ) { Screen::show_message( e.s ); return; }
   _curwin = handle;
-  if( windows() > 1 && bufhandle_vector.handles() > old_handles )
+  if( windows() > 1 && Bufhandle_vector::handles() > old_handles )
     load( windows() + 1 );
   else load();
   }
@@ -156,7 +160,7 @@ void Window_vector::load_file( const int abort_key ) throw()
 
 void Window_vector::split_window() throw()
   {
-  bufhandle_vector.duplicate_handle( _curwin++ );
+  Bufhandle_vector::duplicate_handle( _curwin++ );
   load( windows() + 1 );
   }
 
@@ -165,7 +169,7 @@ void Window_vector::save_file( const int abort_key ) throw()
   {
   const Buffer & buffer = data[_curwin].buffer();
 
-  std::string name = buffer.name();
+  std::string name( buffer.name() );
   if( Menu::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
     return;
   bool done;
@@ -187,7 +191,7 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
   {
   Window & w = data[_curwin];
   Buffer_handle & bh = w.buffer_handle();
-  const bool last_handle = bufhandle_vector.handles( bh.buffer() ) <= 1;
+  const bool last_handle = Bufhandle_vector::handles( bh.buffer() ) <= 1;
   const bool modified = bh.buffer().modified();
   const bool exit_ask = RC::editor_options().exit_ask && !quiet;
 
@@ -201,9 +205,9 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
       }
     if( !abort && modified )
       {
-      std::string name = bh.buffer().name();
+      std::string name( bh.buffer().name() );
       if( exit_ask || !name.size() )
-        while( !Menu::get_filename( "Name of file to save (^C to abort): ", name ) );
+        while( !Menu::get_filename( "Name of file to save (^C to abort): ", name ) ) ;
       if( !name.size() ) return -1;
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error e )
@@ -215,7 +219,7 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
       if( key != 'y' && key != 'Y' ) return -1;
       }
     }
-  bufhandle_vector.delete_handle( _curwin );
+  Bufhandle_vector::delete_handle( _curwin );
   if( _curwin > 0 ) --_curwin;
   return load();
   }
@@ -229,8 +233,7 @@ int Window_vector::close_and_exit( const bool abort ) throw()
   if( key == 'y' || key == 'Y' )
     {
     prev();
-    while( data.size() )
-      { Screen::clock_handler( 0 ); if( close( abort, true ) < 0 ) break; }
+    while( data.size() && close( abort, true ) >= 0 ) ;
     }
   return data.size();
   }
@@ -241,26 +244,26 @@ Window & Window_vector::curwin() throw() { return data[_curwin]; }
 
 void Window_vector::next() throw()
   {
-  const int handles = bufhandle_vector.handles();
+  const int handles = Bufhandle_vector::handles();
   if( handles <= 1 ) return;
   if( _curwin + 1 < windows() ) ++_curwin;
   else
     {
     if( handles <= windows() ) _curwin = 0;
-    else { bufhandle_vector.next(); load(); }
+    else { Bufhandle_vector::next(); load(); }
     }
   }
 
 
 void Window_vector::prev() throw()
   {
-  const int handles = bufhandle_vector.handles();
+  const int handles = Bufhandle_vector::handles();
   if( handles <= 1 ) return;
   if( _curwin > 0 ) --_curwin;
   else
     {
     if( handles <= windows() ) _curwin = windows() - 1;
-    else { bufhandle_vector.prev(); load(); }
+    else { Bufhandle_vector::prev(); load(); }
     }
   }
 
@@ -278,7 +281,12 @@ int Window_vector::windows() throw() { return data.size(); }
 void Window_vector::add_char( unsigned char ch, const bool force ) throw()
   {
   Window & w = data[_curwin];
-  if( read_only( w.buffer() ) ) return;
+  if( w.buffer().options.read_only )
+    {
+    if( ch == ' ' && !force ) w.move_vertical( w.height() - 2 );
+    else read_only( w.buffer() );
+    return;
+    }
 
   if( ch == '\r' && !force ) ch = '\n';
   if( ( force || ch == '\t' || ch == '\n' || ISO_8859::isprint( ch ) ) &&
@@ -414,7 +422,8 @@ void Window_vector::indent_block() throw()
     if( p.col > 0 || p < buffer.eol( p ) )
       {
       if( first_post ) first_post = false; else buffer.force_append();
-      buffer.pputc( p, ' ' ); buffer.pputc( p, ' ' );
+      for( int i = 0; i < RC::editor_options().indent_step; ++i )
+        buffer.pputc( p, ' ' );
       }
     ++p.line;
     }
@@ -435,7 +444,8 @@ void Window_vector::unindent_block() throw()
   bool fail = false;
   while( buffer.bol( p ) < Block::end() && !fail )
     {
-    if( buffer.eol( p ).col > 0 && buffer.to_cursor( buffer.bot( p ) ).col < 2 )
+    if( buffer.eol( p ).col > 0 &&
+        buffer.to_cursor( buffer.bot( p ) ).col < RC::editor_options().indent_step )
       fail = true;
     ++p.line;
     }
@@ -448,7 +458,7 @@ void Window_vector::unindent_block() throw()
   p = Block::begin();
   while( buffer.bol( p ) < Block::end() )
     {
-    const int target_col = buffer.to_cursor( buffer.bot( p ) ).col - 2;
+    const int target_col = buffer.to_cursor( buffer.bot( p ) ).col - RC::editor_options().indent_step;
     if( buffer.eol( p ).col > 0 )
       while( true )
         {
@@ -574,6 +584,37 @@ void Window_vector::reformat() throw()
   }
 
 
+void Window_vector::change_case( const bool to_caps ) throw()
+  {
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
+    return;
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  bool change = false;
+  if( to_caps )
+    for( Point p1 = Block::begin(); p1 < Block::end(); )
+      {
+      const bool in_block = Block::in_block( buffer, p1 );
+      unsigned char ch = buffer.pgetc( p1 );
+      if( in_block && ISO_8859::islower( ch ) )
+        { ch = ISO_8859::toupper( ch ); change = true; }
+      tmp.pputc( p, ch );
+      }
+  else
+    for( Point p1 = Block::begin(); p1 < Block::end(); )
+      {
+      const bool in_block = Block::in_block( buffer, p1 );
+      unsigned char ch = buffer.pgetc( p1 );
+      if( in_block && ISO_8859::isupper( ch ) )
+        { ch = ISO_8859::tolower( ch ); change = true; }
+      tmp.pputc( p, ch );
+      }
+  if( change ) replace_and_repaint( buffer, tmp );
+  else Screen::show_message( "No letters to change in block" );
+  }
+
+
 void Window_vector::encode_base64() throw()
   {
   Buffer & buffer = data[_curwin].buffer();
@@ -655,7 +696,169 @@ void Window_vector::encode_rot1347( const bool mode13 ) throw()
   }
 
 
-void Window_vector::search( const int abort_key, const bool again ) throw()
+void Window_vector::remove_duplicate_lines( const bool back ) throw()
+  {
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+
+  bool change = false;
+  if( back )
+    {
+    int first_line = Block::begin().line;
+    if( Block::begin().col > 0 ) ++first_line;
+    for( int line = Block::end().line - 1; line > first_line; --line )
+      for( Point p( line - 1, 0 ); p.line >= first_line; --p.line )
+        if( buffer.compare_lines( line, p.line ) )
+          {
+          if( change ) buffer.force_append(); else change = true;
+          buffer.pdell( p );
+          --line;
+          }
+    }
+  else
+    {
+    int line = Block::begin().line;
+    if( Block::begin().col > 0 ) ++line;
+    for( ; line < Block::end().line - 1; ++line )
+      for( Point p( Block::end().line - 1, 0 ); p.line > line; --p.line )
+        if( buffer.compare_lines( line, p.line ) )
+          {
+          if( change ) buffer.force_append(); else change = true;
+          buffer.pdell( p );
+          }
+    }
+  if( change )
+    {
+    buffer.reset_appendable();
+    if( Block::valid() )
+      {
+      if( back ) data[_curwin].update_points( Block::end(), true, true );
+      else data[_curwin].update_points( Block::begin(), true, true );
+      }
+    if( RC::editor_options().auto_unmark ) Block::reset();
+    Screen::repaint( &buffer );
+    }
+  else Screen::show_message( "No duplicate lines in block" );
+  }
+
+
+void Window_vector::copyright_update() throw()
+  {
+  static std::vector< std::string > history;
+  const std::string search_string( "\\<Copyright\\>\\+\\[^\\n]\\+\\+\\+\\[0-9]\\*\\<\\[^0-9]" );
+
+  std::string year_string;
+  time_t dummy = time( 0 ); struct tm *t = localtime( &dummy );
+  if( t )
+    {
+    int year = 1900 + t->tm_year;
+    while( year )
+      { year_string.insert( 0, 1, '0' + ( year % 10 ) ); year /= 10; }
+    }
+  history.push_back( year_string );
+  if( Screen::get_string( "Year to add (^C to abort): ", history ) <= 0 )
+    return;
+  year_string = history.back();
+  for( unsigned int i = 0; i < year_string.size(); ++i )
+    if( !std::isdigit( year_string[i] ) )
+      year_string.clear();
+  if( !year_string.size() || year_string[0] == '0' )
+    { Screen::show_message( "Invalid year" ); return; }
+  year_string.insert( 0, "\\<" ); year_string += "\\>";
+  bool ask = true, found = false, modified = false;
+  for( int i = 0; i < Bufhandle_vector::handles(); next(), ++i )
+    {
+    Window & w = data[_curwin];
+    if( w.buffer().options.read_only ) continue;
+    Point p = w.buffer().bof();
+    std::vector< std::string > regex_pieces;
+    while( true )
+      {
+      Point p1 = p;
+      if( !Regex::find( w.buffer(), p1, p, search_string, regex_pieces ) )
+        break;
+      found = true;
+      const Point p0 = w.buffer().bol( p1 );
+      const Point p2 = w.buffer().eol( p );
+      while( w.buffer().pprev( p ) && !w.buffer().piseow( p ) ) ;
+      Basic_buffer tmp( w.buffer(), p0, p );
+      Point pp = tmp.bof();
+      if( p1 < p && w.buffer().piseow( p ) &&
+          !Regex::find( tmp, pp, pp, year_string, regex_pieces ) )
+        {
+        const int rmargin = w.buffer().options.rmargin();
+        pp = tmp.eof();
+        tmp.pputc( pp, ',' );
+        if( tmp.to_cursor( pp ).col + (int)year_string.size() - 4 > rmargin + 1 )
+          {
+          tmp.pputc( pp, '\n' );
+          tmp.pputb( pp, w.buffer(), p0, p1 );
+          }
+        else tmp.pputc( pp, ' ' );
+        for( unsigned int i = 2; i < year_string.size() - 2; ++i )
+          tmp.pputc( pp, year_string[i] );
+        while( p < p2 && std::ispunct( w.buffer()[p] ) && w.buffer().pnext( p ) ) ;
+        while( p < p2 && std::isspace( w.buffer()[p] ) && w.buffer().pnext( p ) ) ;
+        if( p < p2 )
+          {
+          Point pp1 = pp;
+          tmp.pputb( pp, w.buffer(), p, p2 );
+          if( tmp.to_cursor( pp ).col + 1 > rmargin + 1 )
+            {
+            tmp.pputc( pp1, '\n' );
+            tmp.pputb( pp1, w.buffer(), p0, p1 );
+            }
+          else tmp.pputc( pp1, ' ' );
+          }
+        p = p2;
+        w.update_points( p, true, true );
+        int key = 0;
+        if( ask || Screen::kbhit() >= 0 )
+          {
+          key = Screen::show_message( "Update (Y)es (N)o (R)est (^C to abort): ", true );
+          if( key == 3 ) return;
+          if( ask && ( key == 'r' || key == 'R' ) ) ask = false;
+          }
+        if( !ask || key == 'y' || key == 'Y' )
+          {
+          w.buffer().replace( p0, p, tmp, tmp.bof(), tmp.eof() );
+          w.update_points( p );
+          Screen::repaint( &w.buffer() );
+          modified = true;
+          }
+        }
+      }
+    }
+  if( !found ) Screen::show_message( "No copyright notices found" );
+  else if( !modified )
+    Screen::show_message( "No copyright notices have been modified" );
+  else Screen::show_message( "Finished" );
+  }
+
+
+void Window_vector::search_word( const int abort_key ) throw()
+  {
+  const Buffer & buffer = data[_curwin].buffer();
+  Point p1 = data[_curwin].pointer();
+  if( ISO_8859::isalnum_( buffer[p1] ) || buffer.piseow( p1 ) )
+    {
+    Point p2 = p1;
+    while( !buffer.pisbow( p1 ) ) if( !buffer.pprev( p1 ) ) break;
+    while( !buffer.piseow( p2 ) ) if( !buffer.pnext( p2 ) ) break;
+    if( buffer.pisbow( p1 ) && buffer.piseow( p2 ) )
+      {
+      std::string s_ini;
+      buffer.to_string( p1, p2, s_ini );
+      s_ini.insert( 0, "\\<" ); s_ini += "\\>";
+      search( abort_key, false, s_ini );
+      }
+    }
+  }
+
+
+void Window_vector::search( const int abort_key, const bool again,
+                            const std::string & s_ini ) throw()
   {
   static std::vector< std::string > search_history;
   static std::vector< std::string > replace_history;
@@ -667,7 +870,7 @@ void Window_vector::search( const int abort_key, const bool again ) throw()
 
   if( !again || search_history.size() == 0 )
     {
-    search_history.push_back( std::string() );
+    search_history.push_back( s_ini );
     if( Screen::get_string( "Find (^C to abort): ", search_history, abort_key, true ) <= 0 )
       return;
     search_string = search_history.back(); ISO_8859::deescapize( search_string );
@@ -715,7 +918,7 @@ void Window_vector::search( const int abort_key, const bool again ) throw()
     }
   bool found = false, ask = true, found_any = false, wrapped = false;
   bool empty_replace = false;
-  for( int i = (global ? bufhandle_vector.handles() - 1 : 0); i >= 0 && !found; --i )
+  for( int i = (global ? Bufhandle_vector::handles() - 1 : 0); i >= 0 && !found; --i )
     {
     Window & w = data[_curwin];
     std::vector< std::string > regex_pieces;
@@ -774,23 +977,27 @@ void Window_vector::search( const int abort_key, const bool again ) throw()
             break;
             }
           }
-        ++i; Block::restore_block_position(); if( ask || found ) w.repaint();
+        ++i; Block::restore_block_position();
+        if( ask || found ) Screen::repaint( &w.buffer() );
         if( error.size() ) Screen::show_message( error );
         }
       continue;
       }
     if( !ask ) w.repaint();
     if( block ) break;
-    if( global && !wrapped )
+    if( !wrapped )
       {
-      if( backward ) { prev(); p = curwin().buffer().eof(); }
-      else { next(); p = curwin().buffer().bof(); }
-      }
-    if( ( ( global && i == 0 ) || RC::editor_options().search_wrap ) && !wrapped )
-      {
-      if( backward ) p = curwin().buffer().eof();
-      else p = w.buffer().bof();
-      wrapped = true; ++i;
+      if( global )
+        {
+        if( backward ) { prev(); p = curwin().buffer().eof(); }
+        else { next(); p = curwin().buffer().bof(); }
+        }
+      if( ( global && i == 0 ) || ( !global && RC::editor_options().search_wrap ) )
+        {
+        if( backward ) p = curwin().buffer().eof();
+        else p = w.buffer().bof();
+        wrapped = true; ++i;
+        }
       }
     }
   if( !found_any )
