@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,11 +48,16 @@ void redo() throw();
 
 void center_line() throw();
 void reformat() throw();
+void change_case( const bool to_caps ) throw();
 void encode_base64() throw();
 void decode_base64() throw();
 void decode_quoted_printable_utf8( const bool mode_q ) throw();
 void encode_rot1347( const bool mode13 ) throw();
+void remove_duplicate_lines( const bool back = false ) throw();
 
-void search( const int abort_key, const bool again = false ) throw();
+void copyright_update() throw();
+void search_word( const int abort_key ) throw();
+void search( const int abort_key, const bool again = false,
+             const std::string & s_ini = "" ) throw();
 
 } // end namespace Window_vector
